/******************************************************************************
File:    SC_MSG.H
Project: Serial controller developer's library
Purpose: System and controller message descriptors.
Company: Comtrol Corporation
Author:  Craig Harrison
*******************************************************************************
Copyright 1993 Comtrol Corporation. All rights reserved.
Subject to developers license agreement.
*******************************************************************************
Comments:
This file contains SCP and API defined message descriptors.  These describe the
structure of messages that are passed between the system and the controller.
******************************************************************************/



/******************************************************************************
                        Controller Message Descriptors
Controller messages are passed from the SSCI to the SCK through the CtlQ.
Each message has a descriptor associated with it that defines the structure
of the message.  This descriptor can have a maximum size of MSG_LEN words.
All descriptor members must be constructed from defined types contained
in both s_types.h and c_types.h.  The first two bytes of each descriptor
must be the MsgType and ReplyID fields.  The remainder can be defined as
needed by the event function writer.  Controller message descriptors are
named as follows:
   CM_XX_T
where:
   CM stands for Controller Message
   XX is the two digit message type
There is also a define for each controller message which is set to the message
type number.  This define has the form:
   CMTYPE_YYY
where YYY is a name that describes the purpose of the message.
Each CM descriptor in this file will have an associated event function in file
C_EV.C.


                          **** WARNINGS ****
The CMTBL_SIZE define in C_KERNDF.H must be set to the number of controller
messages that exist.
CM_00_T through CM_15_T cannot be defined here, they are SCK defined
descriptors.
Each controller message's event function must be installed with kInstallEvCM().
******************************************************************************/
typedef struct                /* Open line for async message */
{
   UBYTE_T MsgType;           /* message type */
   BYTE_T ReplyID;            /* reply identification */
   UBYTE_T LineNum;           /* line number */
   UBYTE_T Baud;              /* COM_BAUD_XXX flag defined in SC_IPDEF.H */
   UWORD_T OpenType;          /* Open type: COM_TX and/or COM_RX */
   UBYTE_T Parity;            /* parity flag: COM_PAR_NONE, COM_PAR_EVEN,
                                              COM_PAR_ODD */
   UBYTE_T DataBits;          /* num data bits: COM_DATABIT_5, COM_DATABIT_6,
                                                COM_DATABIT_7, COM_DATABIT_8 */
   UBYTE_T StopBits;          /* num stop bits: COM_STOPBIT_1, COM_STOPBIT_15,
                                                COM_STOPBIT_2 */
   UBYTE_T ModemCtl;          /* Modem control lines to turn ON, can be
                                 COM_MDM_RTS or COM_MDM_DTR.  If a flag is
                                 not set that modem line will be OFF. */
   UWORD_T FlowCtl;           /* flow control flags: COM_FLOW_IS, COM_FLOW_OS,
                                 COM_FLOW_IH, COM_FLOW_OH.  At most one input
                                 and one output flag allowed. */
   UWORD_T DetectEn;          /* ESC and SRC detect enables, can be any
                                 combination of the following flags:
                                 COM_DEN_PARITY, COM_DEN_RXOVR, COM_DEN_FRAME,
                                 COM_DEN_BUFOVR, COM_DEN_CD, COM_DEN_CTS,
                                 COM_DEN_BKABORT. */
} CM_16_T;
#define CMTYPE_OPENASYNC 16

typedef struct                /* Reconfigure line for async message.  This can
                                 be used to change async config after the
                                 open line message has been issued. */
{
   UBYTE_T MsgType;           /* message type */
   BYTE_T ReplyID;            /* reply identification */
   UBYTE_T LineNum;           /* line number */
   UBYTE_T Parity;            /* parity flag: COM_PAR_NONE, COM_PAR_EVEN,
                                              COM_PAR_ODD */
   UWORD_T Baud;              /* baud rate flag defined in SC_IPDEF.H */
   UBYTE_T DataBits;          /* num data bits: COM_DATABIT_5, COM_DATABIT_6,
                                                COM_DATABIT_7, COM_DATABIT_8 */
   UBYTE_T StopBits;          /* num stop bits: COM_STOPBIT_1, COM_STOPBIT_15,
                                                COM_STOPBIT_2 */
   UWORD_T FlowCtl;           /* flow control flags: COM_FLOW_IS, COM_FLOW_OS,
                                 COM_FLOW_IH, COM_FLOW_OH.  At most one input
                                 and one output flag allowed. */
   UWORD_T DetectEn;          /* ESC and SRC detect enables, can be any
                                 combination of the following flags:
                                 COM_DEN_PARITY, COM_DEN_RXOVR, COM_DEN_FRAME,
                                 COM_DEN_BUFOVR, COM_DEN_CD, COM_DEN_CTS,
                                 COM_DEN_BKABORT. */
} CM_17_T;
#define CMTYPE_RECFGASYNC 17

typedef struct                /* Change state of modem output lines */
{
   UBYTE_T MsgType;           /* message type */
   BYTE_T ReplyID;            /* reply identification */
   UBYTE_T LineNum;           /* line number */
   UBYTE_T RTSState;          /* state of RTS line: ON, OFF, or NOCHANGE */
   UBYTE_T DTRState;          /* state of DTR line: ON, OFF, or NOCHANGE */
} CM_18_T;
#define CMTYPE_CHGMDM 18
#define NOCHANGE 2            /* no change in state */

typedef struct                /* Get state of all modem lines */
{
   UBYTE_T MsgType;           /* message type */
   BYTE_T ReplyID;            /* reply identification */
   UBYTE_T LineNum;           /* line number */
} CM_19_T;
#define CMTYPE_GETMDM 19

typedef struct                /* Close line for async message */
{
   UBYTE_T MsgType;           /* message type */
   BYTE_T ReplyID;            /* reply identification */
   UBYTE_T LineNum;           /* line number */
   UBYTE_T ModemClr;          /* Modem control lines to turn OFF, can be
                                 COM_MDM_RTS or COM_MDM_DTR.  If a flag is
                                 not set that modem line is not changed. */
} CM_20_T;
#define CMTYPE_CLOSEASYNC 20

typedef struct                /* Reserved message */
{
   UBYTE_T MsgType;           /* message type */
   BYTE_T ReplyID;            /* reply identification */
} CM_21_T;
#define CMTYPE_RES21 21

typedef struct                /* Reserved message */
{
   UBYTE_T MsgType;           /* message type */
   BYTE_T ReplyID;            /* reply identification */
} CM_22_T;
#define CMTYPE_RES22 22

typedef struct                /* Get Tx data count message */
{
   UBYTE_T MsgType;           /* message type */
   BYTE_T ReplyID;            /* reply identification */
   UBYTE_T LineNum;           /* line number */
} CM_23_T;
#define CMTYPE_GETTXCNT 23

typedef struct                /* Get Rx data count message */
{
   UBYTE_T MsgType;           /* message type */
   BYTE_T ReplyID;            /* reply identification */
   UBYTE_T LineNum;           /* line number */
} CM_24_T;
#define CMTYPE_GETRXCNT 24

typedef struct                /* Flush Tx and/or Rx data for a line  */
{
   UBYTE_T MsgType;           /* message type */
   BYTE_T ReplyID;            /* reply identification */
   UBYTE_T LineNum;           /* line number */
   UBYTE_T Flags;             /* flags for type of flush, use any combination
                                 of COM_TXIP, COM_TXLOC, COM_RXIP, and
                                 COM_RXLOC. */
} CM_25_T;
#define CMTYPE_FLUSH 25

typedef struct                /* Reserved message */
{
   UBYTE_T MsgType;           /* message type */
   BYTE_T ReplyID;            /* reply identification */
} CM_26_T;
#define CMTYPE_RES26 26


typedef struct                /* Send a break out a line */
{
   UBYTE_T MsgType;           /* message type */
   BYTE_T ReplyID;            /* reply identification */
   UBYTE_T LineNum;           /* line number */
   UBYTE_T Fill;              /* keep it on even boundaries */
   WORD_T BkTime;    /* time in milliseconds for break to be sent, 32767 max */
} CM_27_T;
#define CMTYPE_SENDBREAK 27


/******************************************************************************
                        System Message Descriptors
System messages are passed from the SCK to the SSCI through the SysQ.
Each message has a descriptor associated with it that defines the structure
of the message.  This descriptor can have a maximum size of MSG_LEN words.
All descriptor members must be constructed from defined types contained
in both s_types.h and c_types.h.  The first two bytes of each descriptor
must be the MsgType and ReplyID fields.  The remainder can be defined as
needed by the event function writer.  System message descriptors are
named as follows:
   SM_XX_T
where:
   SM stands for System Message
   XX is the two digit message type
There is also a define for each system message which is set to the message
type number.  This define has the form:
   SMTYPE_YYY
where YYY is a name that describes the purpose of the message.


                          **** WARNINGS ****
SM_00_T through SM_15_T cannot be defined here, they are SSCI defined
descriptors.
******************************************************************************/
typedef struct                /* Modem line (CD or CTS) change of state  */
{
   UBYTE_T MsgType;           /* message type */
   BYTE_T ReplyID;            /* reply identification */
   UBYTE_T LineNum;           /* line number */
   UBYTE_T MdmChange;         /* modem input lines which changed state using
                                 COM_MDM_XXX flags */
   UBYTE_T MdmState;          /* current state of modem lines using COM_MDM_XXX
                                 flags */
} SM_16_T;
#define SMTYPE_MDMCHANGE 16

typedef struct                /* Break occurred message */
{
   UBYTE_T MsgType;           /* message type */
   BYTE_T ReplyID;            /* reply identification */
   UBYTE_T LineNum;           /* line number */
} SM_17_T;
#define SMTYPE_BREAK 17

typedef struct                /* Rx error message */
{
   UBYTE_T MsgType;           /* message type */
   BYTE_T ReplyID;            /* reply identification */
   UBYTE_T LineNum;           /* line number */
   UBYTE_T Status;            /* error status */
} SM_18_T;
#define SMTYPE_RXERR 18

