/******************************************************************************
File:    SC_IP.H
Project: Serial controller developers library
Purpose: Interprocessor communication defines and descriptors.
Company: Comtrol Corporation
Author:  Craig Harrison
*******************************************************************************
Copyright 1993 Comtrol Corporation. All rights reserved.
Subject to developers license agreement.
******************************************************************************/

#ifndef SC_IP
#define SC_IP


/******************************************************************************
                           Memory Map Defines
FIXDPM_OFF:  Offset of the fixed DPM area from the start of DPM.  This is
        the same as the absolute offset of the firmware user area from the
        beginning of DPM.
******************************************************************************/
#if defined SMARTH
   #define FIXDPM_OFF 0
#elif defined HOSTESS186
   #define FIXDPM_OFF 0xb80
#elif defined HOSTESSi
   #define FIXDPM_OFF 0xb80
#else
   #error CTLTYPE not defined to a known controller type
#endif

/*****************************************************************************-
                        Line Status Flags (UWORD_T)
These are used by the Status member of the line descriptor, LINE_T.
******************************************************************************/
#define LN_OPENTX       0x0001   /* line open for transmit */
#define LN_OPENRX       0x0002   /* line open for receive */
#define LN_OPEN         0x0004   /* line open */
#define LN_ISFLOWEN     0x0008   /* input software flow contol enabled */
#define LN_OSFLOWEN     0x0010   /* output software flow control enabled */
#define LN_IHFLOWEN     0x0020   /* input hardware flow control enabled */
#define LN_OHFLOWEN     0x0040   /* output hardware flow control enabled */
#define LN_IFLOWON      0x0080   /* input data flow is ON */
#define LN_OFLOWON      0x0100   /* output data flow is ON */
#define LN_OXANY        0x0200   /* output flow restart on ANY Rx byte */
#define LN_IFLOWSTOPCK  0x0400   /* time to check for input data flow stop */
#define LN_SENDBK       0x0800   /* a break currently being sent on the line */
#define LN_RECBK        0x1000   /* a break being received on the line */

/*****************************************************************************-
                        Communications Parameter Flags
These are used in CMs (controller messages) and SMs (system messages) and
their event functions.
Note: Many of these flags are defined to other defines, this done to improve
      performance when interpreting them.
Note: The baud rate flags are located in SC_IPDEF.H.
******************************************************************************/
/* TX and RX identifier flags (UWORD_T) */
#define COM_TX       LN_OPENTX         /* transmit (0x0001) */
#define COM_RX       LN_OPENRX         /* receive (0x0002) */
#define COM_TXIP     0x0004            /* transmit interprocessor side */
#define COM_TXLOC    0x0008            /* transmit local side */
#define COM_RXIP     0x0010            /* receive interprocessor side */
#define COM_RXLOC    0x0020            /* receive local side */

/* ES (external status) state flags (UWORD_T).  Those applying to modem
   control lines are named COM_MDM_XXX, the rest are COM_ES_XXX.  If only
   the COM_MDM flags are referred to they can be considered UBYTE_T. */
#define COM_MDM_DSR   0x01             /* data set ready */
#define COM_MDM_RTS   RTS              /* request to send (0x02) */
#define COM_MDM_RI    0x04             /* ring indicator */
#define COM_MDM_CD    CD               /* carrier detect (0x08) */
#define COM_MDM_CTS   CTS              /* clear to send (0x20) */
#define COM_MDM_DTR   DTR              /* data terminal ready (0x80) */
#define COM_ES_SYNCHUNT (SYNCHUNT << 8)  /* sync/hunt (bisync, SDLC) (0x1000) */
#define COM_ES_TXUNDRUN (TXUNDRUN << 8)  /* Tx underrun/EOM (SDLC) (0x4000) */
#define COM_ES_BKABORT  (BKABORT << 8)   /* break/abort (async, SDLC) (0x8000) */

/* Stop bit flags (UBYTE_T) */
#define COM_STOPBIT_1  ONE_STOP_BIT    /* 1 stop bit (0x04) */
#define COM_STOPBIT_15 ONEH_STOP_BITS  /* 1.5 stop bits (0x08) */
#define COM_STOPBIT_2  TWO_STOP_BITS   /* 2 stop bits (0x0c) */

/* Data bit flags (UBYTE_T) */
#define COM_DATABIT_5  TX5_BIT_CHAR    /* 5 data bits (0x00) */
#define COM_DATABIT_6  TX6_BIT_CHAR    /* 6 data bits (0x40) */
#define COM_DATABIT_7  TX7_BIT_CHAR    /* 7 data bits (0x20) */
#define COM_DATABIT_8  TX8_BIT_CHAR    /* 8 data bits (0x60) */

/* Parity flags (UBYTE_T) */
#define COM_PAR_NONE   0               /* no parity */
#if QNX
#define COM_PAR_EVEN   (PARITY_ENABLE | cPARITY_EVEN) /* even parity (0x03) */
#define COM_PAR_ODD    (PARITY_ENABLE | cPARITY_ODD)  /* odd parity (0x01) */
#else
#define COM_PAR_EVEN   (PARITY_ENABLE | PARITY_EVEN) /* even parity (0x03) */
#define COM_PAR_ODD    (PARITY_ENABLE | PARITY_ODD)  /* odd parity (0x01) */
#endif

/* Break flag (UBYTE_T) */
#define COM_SEND_BK    BREAK           /* send break (0x10) */

/* Flow control flags (UWORD_T) */
#define COM_FLOW_NONE  0
#define COM_FLOW_IS    LN_ISFLOWEN     /* input software flow control */
#define COM_FLOW_IH    LN_IHFLOWEN     /* input hardware flow control */
#define COM_FLOW_OS    LN_OSFLOWEN     /* output software flow control */
#define COM_FLOW_OH    LN_OHFLOWEN     /* output hardware flow control */
#define COM_FLOW_OXANY LN_OXANY        /* restart output on any Rx char */

/* Detection enable flags (UWORD_T) */
      /* SRC (Special Receive Condition) detection enables.  Support for these
         must exist in the SRC event function(s). */
#define COM_DEN_BUFOVR 0x0100    /* enable Rx buf overflow detection */
#define COM_DEN_PARITY 0x1000    /* enable parity error detection */
#define COM_DEN_RXOVR  0x2000    /* enable Rx channel overrun detection */
#define COM_DEN_FRAME  0x4000    /* enable framing (async) or CRC (sync) det */
#define COM_DEN_EOF    0x8000    /* enable EOF (SDLC) detection */

      /* ESC (External Status Change) detection enables.  Support for these
         must exist in the ESC event function(s). Note that COM_DEN_DSR,
         COM_DEN_RI, COM_DEN_CD, and COM_DEN_CTS were chosen to match
         COM_MDM_DSR, COM_MDM_RI, COM_MDM_CD, and COM_MDM_CTS respectively.
         This was done for speed in coding kGetExtStatChg().  Changing these
         flags in ports to other USARTs will break kGetExtStatChg(). */
#define COM_DEN_DSR      0x0001       /* enable DSR change detection */
#define COM_DEN_RI       0x0004       /* enable RI change detection */
#define COM_DEN_CD       CD_IE        /* enable CD change detection (0x0008) */
#define COM_DEN_SYNCHUNT SYNCHUNT_IE  /* enable SYNC/HUNT detection (0x0010) */
#define COM_DEN_CTS      CTS_IE       /* enable CTS change detection (0x0020)*/
#define COM_DEN_TXUNDRUN TXUNDRUN_IE  /* enab Tx Underrun/EOM detect (0x0040)*/
#define COM_DEN_BKABORT  BKABORT_IE   /* enable break/abort detection (0x0080*/


/* SRC (special receive condition) state flags (UBYTE_T) */
#define COM_SRC_BUFOVR 0x01            /* Rx buf overflow because out of SDBs*/
#define COM_SRC_PARITY PARITY_ERR      /* parity error (async) (0x10) */
#define COM_SRC_RXOVR  OVRRUN_ERR      /* Rx overrun error (all modes) (0x20)*/
#define COM_SRC_FRAME  FRAME_ERR       /* framing error (all modes) (0x40) */
#define COM_SRC_EOF    EOFRM           /* end of frame (SDLC) (0x80) */


/*****************************************************************************-
                        Firmware User Area Descriptor
This area is defined and initialized by the firmware.  After the SCK
is started, the firmware can modify only the IIFlag, IIType, and IICnt members,
which indicate invalid interrupts have occurred.  The firmware automatically
recovers from invalid interrupts and returns control to the SCK.  The
SCK modifies only the IFlag and Heartbeat members.
******************************************************************************/
typedef struct
{
   UWORD_T IFlag;                   /* processor interaction flag, def below */
   UWORD_T BootFlag;                /* boot/activity flag */
   UWORD_T CfgMap;                  /* configuration map */
   BYTE_T FwRelease[8];             /* firmware release number */
   BYTE_T SwRelease[8];             /* control program release number */
   ULONG_T DRAMMap1;                /* DRAM map */
   ULONG_T DRAMMap2;                /* DRAM map */
   ULONG_T SCCMap;                  /* SCC map */
   ULONG_T ControllerID;            /* controller ID */
   BYTE_T IIFlag;                   /* invalid interrupt flag */
   BYTE_T IIType;                   /* invalid interrupt type */
   UWORD_T IICnt;                   /* invalid interrupt count */
   ULONG_T Heartbeat;               /* heartbeat counter */
   BYTE_T Cmd;                      /* firmware utility command */
   BYTE_T Status;                   /* firmware utility status */
   BYTE_T Msg[16];                  /* firmware utility message buffer */
   BYTE_T Reserved1[0x40];          /* reserved for future use */
} FIRMUSER_T;

/* Interaction flag defines */
#define IF_RDY    0x55aa            /* ready */
#define IF_NRDY   0xaa55            /* not ready */
#define IF_BK     0x4b42            /* SCP or SCK break */
#define IF_INITERR 0x4945           /* SCK initialization error */

/* Memory map defines for DRAMMap1 and DRAMMap2 */
#define NORMMEM_128K    0x0003      /* 128K normal mode memory */
#define NORMMEM_640K    0x03ff      /* 640K normal mode memory */
#define NORMMEM_1M      0xffff      /* 1M normal mode memory */
#define EXTMEM_0        0           /* no extended mode memory */
#define EXTMEM_1M       0x0003      /* 1M extended mode memory */
#define EXTMEM_7M       0x3fff      /* 7M extended mode memory */


/*****************************************************************************
                        Stack Probe Descriptor
This descriptor is used when a stack probe detects that a task stack has
overflowed.  C_START.ASM has hard coded offsets into this descriptor, so
any change here requires a change there.  Stack probes are only done when
compiling with the development libraries by setting tunable TRACE to 1.
******************************************************************************/
typedef struct
{
   UBYTE_T State;                      /* state of stack probes, defined below */
   UBYTE_T Prio;                       /* priority of task overflowing stk */
   UWORD_T *Ptr;                       /* overflowed stk ptr address */
   UWORD_T *ProbeP;                    /* lowest allowed stk ptr address */
} STKPROBE_T;

#define STKPRB_NOERR 0                 /* no stack overflows */
#define STKPRB_INTCURERR 1             /* interrupt - current task overflow */
#define STKPRB_INTNEWERR 2             /* interrupt - new task generates overflow */
#define STKPRB_SCHCURERR 3             /* schedular - current task overflow */
#define STKPRB_SCHNEWERR 4             /* schedular - new task overflow */


/*****************************************************************************-
                        Message Reply Buffer Descriptor
Used for interprocessor (SSCI - SCK) message passing.  This is used by a
message consumer to send a reply back to the message producer.  The REPLYBUF
is always named for the consumer (SSCK or SCK) of the message that requested
the reply.
******************************************************************************/
#define REPLY_LEN 3                    /* length of reply buffer in words */
typedef struct
{
   UWORD_T Buf[REPLY_LEN];             /* reply buffer */
} REPLYBUF_T;


/*****************************************************************************-
                        Message Queue Descriptor
Used for interprocessor (SSCI - SCK) message passing.  The MSGQ is always
named for the consumer (SSCI or SCK) of the message.  Replys to messages
are sent in the REPLYBUF_T member of the same MSGQ_T descriptor that
contained the original message.
******************************************************************************/
#define MSGQ_SIZE 32              /* number messages in queue buffer */
#define MSG_LEN   8               /* length of message in words */
#define REPLY_SIZE 24             /* number of reply buffers */
#define REPLYSTAT_AVAIL 0         /* reply buffer is available for use */
#define REPLYSTAT_WAIT  1         /* msg producer waiting for reply */
#define REPLYSTAT_RDY   2         /* msg consumer has put reply in reply buf */
typedef struct
{
   BOOL_T Processing;               /* TRUE if consumer is processing msgs */
   BOOL_T Fill;                     /* keep it on even boundaries */
   WORD_T In;                       /* index into queue, add message here */
   WORD_T Out;                      /* index into queue, remove message here */
   UWORD_T Buf[MSGQ_SIZE][MSG_LEN]; /* message buffer */
   UBYTE_T ReplyStat[REPLY_SIZE];   /* status of each reply buffer */
   REPLYBUF_T Reply[REPLY_SIZE];    /* reply buffers */
} MSGQ_T;


/*****************************************************************************-
                        Interrupt Flag Groups (IFG)
IFGs are used for interprocessor interrupts.  When a flag is changed by the
producer, the consumer is interrupted unless it is still processing
flags from a previous interrupt.  Each flag group has 16 flags (bits) that can
be used to signal any event desired to the other processor.
******************************************************************************/
#define IFG_SIZE 8                  /* number of interrupt flag groups */
typedef struct
{
   UBYTE_T Processing; /* bit mask of flag groups being processed by consumer*/
   UBYTE_T Fill;                    /* keep things on even boundaries */
   UWORD_T Grp[IFG_SIZE];           /* flag groups */
} IFG_T;


/*****************************************************************************-
                     Interprocessor Interrupt Descriptor
******************************************************************************/
typedef struct
{
   BYTE_T Type;                     /* interrupt type, defined below */
   BOOL_T ISRBusy;                  /* consumer ISR is busy */
   UWORD_T IntMissCnt;              /* count of missed interrupts */
   MSGQ_T MsgQ;                     /* message queue */
   IFG_T FlagGrp;                   /* flag groups */
} IPINT_T;
#define INT_FG0 0                   /* flag group 0 interrupt type */
#define INT_FG1 1                   /* flag group 1 interrupt type */
#define INT_FG2 2                   /* flag group 2 interrupt type */
#define INT_FG3 3                   /* flag group 3 interrupt type */
#define INT_FG4 4                   /* flag group 4 interrupt type */
#define INT_FG5 5                   /* flag group 5 interrupt type */
#define INT_FG6 6                   /* flag group 6 interrupt type */
#define INT_FG7 7                   /* flag group 7 interrupt type */
#define INT_MSG 8                   /* message in MSGQ interrupt type */
#define INT_TIMEOUT 9         /* producer timed out waiting for int to clear */


/*****************************************************************************-
                        Status Flag Group (SFG) Size
The number of Status Flag Groups. Each group has 16 flags (bits) that can be
used to indicate any status desired to the other processor or other tasks.
******************************************************************************/
#define SFG_SIZE 16                    /* number of SFGs */


/*****************************************************************************-
                        Data Buffer Control Block
One of these is used to describe each SDB (shared data buffer).  The address
of the data buffer is stored as (offset from the start of DPM)/16.

                        Shared Data Buffer (SDB) Pool
This is a pool of data buffers in dual port memory that can be allocated/freed
by either processor.  The SDB pool is maintained by an array of DBCBs in the
fixed DPM area, called kSDBTbl[], but the buffers they describe can be anywhere
in DPM.  Restrictions are that no single buffer can cross a 16K boundary
(this done to facilitate sliding windows), the buffer must start on a 16 byte
boundary, and all SDBs must be located in dual port memory.  SDBs are always
identified using their index in kSDBTbl[], which is referred to as the SDB
handle.  Initially all SDBs are linked together onto a free pool, the first
SDB of which is stored in kSDBFree.  Since either processor can access the
free pool, exclusive access must first be obtained with kGetDPAccess() before
allocating or freeing SDBs.

SDB's should never be operated on by directly accessing the DBCB_T members.
Instead use functions kSetSDBDataCnt(), kGetSDBDataCnt(), kSetSDBNext(),
kGetSDBNext(), kSetSDBPrev(), kGetSDBPrev(), and kGetSDBBuf().
******************************************************************************/
typedef struct
{
   WORD_T Count;              /* total number of data bytes contained in Buf */
   WORD_T Next;               /* handle of next data buffer on list */
   WORD_T Prev;               /* handle of previous data buffer on list */
   UWORD_T BufOff;            /* data buf offset from start of DPM in paragraphs*/
} DBCB_T;
#define NULLSDB -1            /* null SDB, indicates no SDB exists or avail */


/******************************************************************************
                           Operation Mode Flags
These flags are used to describe the modes of operation for each serial
line.  They are set in the Modes member of the line descriptor.  They are
intended to be fixed modes of operation that cannot be changed at run time.

The flags which control Rx and Tx operation are organized into the following
groups:
   Group 1:  MODE_TX_BG, MODE_TX_INT, MODE_TX_DMA  
   Group 2:  MODE_TX_CTSFLOW, MODE_TX_DSRFLOW    
   Group 3:  MODE_RX_BG, MODE_RX_INT, MODE_RX_DMA
   Group 4:  MODE_RX_AGE
   Group 5:  MODE_RX_RTSFLOW, MODE_RX_DTRFLOW
   Group 6:  MODE_RX_SENDINT

The LINEXX_TXMODE defines in SC_KERNDF.H can be set to either MODE_NONE or
the OR of one flag from each of groups 1 and 2.

The LINEXX_RXMODE defines in SC_KERNDF.H can be set to either MODE_NONE or
the OR of one flag from each of groups 3, 4, 5, and 6.


                           ***** WARNINGS *****

Do not set the LINEXX_TXMODE or LINE_XX_RXMODE defines to more than one flag
from each of the groups defined above.  For example, the following define
is incorrect:
   #define LINE00_TXMODE MODE_TX_BG | MODE_TX_INT
because it uses two flags from group 1.
******************************************************************************/
#define MODE_NONE          0x0000    /* line not used */
#define MODE_TX_BG         0x0001    /* background Tx mode */
#define MODE_TX_INT        0x0002    /* interrupt driven Tx mode */
#define MODE_TX_DMA        0x0004    /* DMA Tx mode */
#define MODE_TX_CTSFLOW    0x0008    /* use CTS for output hardware flow ctl */
#define MODE_TX_DSRFLOW    0x0010    /* use DSR for output hardware flow ctl */
#define MODE_RX_INT        0x0020    /* interrupt driven Rx mode */
#define MODE_RX_BG         0x0040    /* background Rx mode */
#define MODE_RX_DMA        0x0080    /* DMA Rx mode */
#define MODE_RX_AGE        0x0100    /* aging of Rx SDBs enabled (use only with
                                        MODE_RX_INT or MODE_RX_BG) */
#define MODE_RX_RTSFLOW    0x0200    /* use RTS for input hardware flow ctl */
#define MODE_RX_DTRFLOW    0x0400    /* use DTR for input hardware flow ctl */
#define MODE_RX_SENDINT    0x0800 /*interrupt SSCI when sending Rx SDB to SSCI*/


/******************************************************************************
                  SCK Defined Controller Message Descriptors
Controller messages are passed from the SSCI to the SCK through the CtlQ.
Each message has a descriptor associated with it that defines the structure
of the message.  This descriptor can have a maximum size of MSG_LEN words.
All descriptor members must be constructed from defined types contained
in both s_types.h and c_types.h.  The first two bytes of each descriptor
must be the MsgType and ReplyID fields.  The remainder can be defined as
needed by the event function writer.  Controller message descriptors are named as follows:
   CM_XX_T
where:
   CM stands for Controller Message
   XX is the two digit message type
There is also a define for each controller message which is set to the message
type number.  This define has the form:
   CMTYPE_YYY
where YYY is a name that describes the purpose of the message.

Controller message types 00 through 15 are reserved by the SCK, and their
descriptors are defined below.  The developer may add additional CM
event functions in file C_EV.C, with associated descriptors in SC_MSG.H.
SCP defined CM event functions must be  installed with kevCMInstall().
******************************************************************************/
#define NULLREPLY -1              /* null reply ID, means no reply requested */

typedef struct                         /* Null message */
{
   UBYTE_T MsgType;                    /* message type */
   BYTE_T ReplyID;                     /* reply identification */
   UWORD_T Buf[(MSG_LEN - 1)];         /* space for a message */
} CM_00_T;
#define CMTYPE_NULL 0

typedef struct                         /* Loop message */
{
   UBYTE_T MsgType;                    /* message type */
   BYTE_T ReplyID;                     /* reply identification */
   UWORD_T Buf[(MSG_LEN - 1)];         /* space for a message */
} CM_01_T;
#define CMTYPE_LOOP 1

typedef struct                /* Set number of serial lines supported msg */
{
   UBYTE_T MsgType;           /* message type */
   BYTE_T ReplyID;            /* reply identification */
   UBYTE_T NumLines;          /* number of serial lines supported */
} CM_02_T;
#define CMTYPE_SETNUMLN 2

typedef struct                         /* Reserved message */
{
   UBYTE_T MsgType;                    /* message type */
   BYTE_T ReplyID;                     /* reply identification */
   BYTE_T SysIntFlag;                  /* 0 turns OFF, 1 turns ON */
} CM_03_T;
#define CMTYPE_SYSINTFLAG 3

typedef struct                         /* Reserved message */
{
   UBYTE_T MsgType;                    /* message type */
   BYTE_T ReplyID;                     /* reply identification */
   UWORD_T Buf[(MSG_LEN - 1)];         /* space for a message */
} CM_04_T;
#define CMTYPE_RESERVED4 4

typedef struct                         /* Reserved message */
{
   UBYTE_T MsgType;                    /* message type */
   BYTE_T ReplyID;                     /* reply identification */
   UWORD_T Buf[(MSG_LEN - 1)];         /* space for a message */
} CM_05_T;
#define CMTYPE_RESERVED5 5

typedef struct                         /* Reserved message */
{
   UBYTE_T MsgType;                    /* message type */
   BYTE_T ReplyID;                     /* reply identification */
   UWORD_T Buf[(MSG_LEN - 1)];         /* space for a message */
} CM_06_T;
#define CMTYPE_RESERVED6 6

typedef struct                         /* Reserved message */
{
   UBYTE_T MsgType;                    /* message type */
   BYTE_T ReplyID;                     /* reply identification */
   UWORD_T Buf[(MSG_LEN - 1)];         /* space for a message */
} CM_07_T;
#define CMTYPE_RESERVED7 7

typedef struct                         /* Reserved message */
{
   UBYTE_T MsgType;                    /* message type */
   BYTE_T ReplyID;                     /* reply identification */
   UWORD_T Buf[(MSG_LEN - 1)];         /* space for a message */
} CM_08_T;
#define CMTYPE_RESERVED8 8

typedef struct                         /* Reserved message */
{
   UBYTE_T MsgType;                    /* message type */
   BYTE_T ReplyID;                     /* reply identification */
   UWORD_T Buf[(MSG_LEN - 1)];         /* space for a message */
} CM_09_T;
#define CMTYPE_RESERVED9 9

typedef struct                         /* Reserved message */
{
   UBYTE_T MsgType;                    /* message type */
   BYTE_T ReplyID;                     /* reply identification */
   UWORD_T Buf[(MSG_LEN - 1)];         /* space for a message */
} CM_10_T;
#define CMTYPE_RESERVED10 10

typedef struct                         /* Reserved message */
{
   UBYTE_T MsgType;                    /* message type */
   BYTE_T ReplyID;                     /* reply identification */
   UWORD_T Buf[(MSG_LEN - 1)];         /* space for a message */
} CM_11_T;
#define CMTYPE_RESERVED11 11

typedef struct                         /* Reserved message */
{
   UBYTE_T MsgType;                    /* message type */
   BYTE_T ReplyID;                     /* reply identification */
   UWORD_T Buf[(MSG_LEN - 1)];         /* space for a message */
} CM_12_T;
#define CMTYPE_RESERVED12 12

typedef struct                         /* Reserved message */
{
   UBYTE_T MsgType;                    /* message type */
   BYTE_T ReplyID;                     /* reply identification */
   UWORD_T Buf[(MSG_LEN - 1)];         /* space for a message */
} CM_13_T;
#define CMTYPE_RESERVED13 13

typedef struct                         /* Reserved message */
{
   UBYTE_T MsgType;                    /* message type */
   BYTE_T ReplyID;                     /* reply identification */
   UWORD_T Buf[(MSG_LEN - 1)];         /* space for a message */
} CM_14_T;
#define CMTYPE_RESERVED14 14

typedef struct                         /* Reserved message */
{
   UBYTE_T MsgType;                    /* message type */
   BYTE_T ReplyID;                     /* reply identification */
   UWORD_T Buf[(MSG_LEN - 1)];         /* space for a message */
} CM_15_T;
#define CMTYPE_RESERVED15 15


/******************************************************************************
                     SSCI Defined System Message Descriptors
System messages are passed from the SCK to the SSCI through the SysQ.
Each message has a descriptor associated with it that defines the structure
of the message.  This descriptor can have a maximum size of MSG_LEN words.
All descriptor members must be constructed from defined types contained
in both s_types.h and c_types.h.  The first two bytes of each descriptor
must be the MsgType and ReplyID fields.  The remainder can be defined as
needed by the event function writer.  System message descriptors are
named as follows:
   SM_XX_T
where:
   SM stands for System Message
   XX is the two digit message type
There is also a define for each system message which is set to the message
type number.  This define has the form:
   SMTYPE_YYY
where YYY is a name that describes the purpose of the message.

System messages 00 through 15 are reserved by the SCK, and their
descriptors are defined below.  The developer may add additional SM
descriptors in SC_MSG.H.
******************************************************************************/
typedef struct                         /* Null message */
{
   UBYTE_T MsgType;                    /* message type */
   BYTE_T ReplyID;                     /* reply identification */
   UWORD_T Buf[(MSG_LEN - 1)];         /* space for a message */
} SM_00_T;
#define SMTYPE_NULL 0

typedef struct                         /* Loop message */
{
   UBYTE_T MsgType;                    /* message type */
   BYTE_T ReplyID;                     /* reply identification */
   UWORD_T Buf[(MSG_LEN - 1)];         /* space for a message */
} SM_01_T;
#define SMTYPE_LOOP 1

typedef struct                         /* Reserved message */
{
   UBYTE_T MsgType;                    /* message type */
   BYTE_T ReplyID;                     /* reply identification */
   UWORD_T Buf[(MSG_LEN - 1)];         /* space for a message */
} SM_02_T;
#define SMTYPE_RESERVED2 2

typedef struct                         /* Reserved message */
{
   UBYTE_T MsgType;                    /* message type */
   BYTE_T ReplyID;                     /* reply identification */
   UWORD_T Buf[(MSG_LEN - 1)];         /* space for a message */
} SM_03_T;
#define SMTYPE_RESERVED3 3

typedef struct                         /* Reserved message */
{
   UBYTE_T MsgType;                    /* message type */
   BYTE_T ReplyID;                     /* reply identification */
   UWORD_T Buf[(MSG_LEN - 1)];         /* space for a message */
} SM_04_T;
#define SMTYPE_RESERVED4 4

typedef struct                         /* Reserved message */
{
   UBYTE_T MsgType;                    /* message type */
   BYTE_T ReplyID;                     /* reply identification */
   UWORD_T Buf[(MSG_LEN - 1)];         /* space for a message */
} SM_05_T;
#define SMTYPE_RESERVED5 5

typedef struct                         /* Reserved message */
{
   UBYTE_T MsgType;                    /* message type */
   BYTE_T ReplyID;                     /* reply identification */
   UWORD_T Buf[(MSG_LEN - 1)];         /* space for a message */
} SM_06_T;
#define SMTYPE_RESERVED6 6

typedef struct                         /* Reserved message */
{
   UBYTE_T MsgType;                    /* message type */
   BYTE_T ReplyID;                     /* reply identification */
   UWORD_T Buf[(MSG_LEN - 1)];         /* space for a message */
} SM_07_T;
#define SMTYPE_RESERVED7 7

typedef struct                         /* Reserved message */
{
   UBYTE_T MsgType;                    /* message type */
   BYTE_T ReplyID;                     /* reply identification */
   UWORD_T Buf[(MSG_LEN - 1)];         /* space for a message */
} SM_08_T;
#define SMTYPE_RESERVED8 8

typedef struct                         /* Reserved message */
{
   UBYTE_T MsgType;                    /* message type */
   BYTE_T ReplyID;                     /* reply identification */
   UWORD_T Buf[(MSG_LEN - 1)];         /* space for a message */
} SM_09_T;
#define SMTYPE_RESERVED9 9

typedef struct                         /* Reserved message */
{
   UBYTE_T MsgType;                    /* message type */
   BYTE_T ReplyID;                     /* reply identification */
   UWORD_T Buf[(MSG_LEN - 1)];         /* space for a message */
} SM_10_T;
#define SMTYPE_RESERVED10 10

typedef struct                         /* Reserved message */
{
   UBYTE_T MsgType;                    /* message type */
   BYTE_T ReplyID;                     /* reply identification */
   UWORD_T Buf[(MSG_LEN - 1)];         /* space for a message */
} SM_11_T;
#define SMTYPE_RESERVED11 11

typedef struct                         /* Reserved message */
{
   UBYTE_T MsgType;                    /* message type */
   BYTE_T ReplyID;                     /* reply identification */
   UWORD_T Buf[(MSG_LEN - 1)];         /* space for a message */
} SM_12_T;
#define SMTYPE_RESERVED12 12

typedef struct                         /* Reserved message */
{
   UBYTE_T MsgType;                    /* message type */
   BYTE_T ReplyID;                     /* reply identification */
   UWORD_T Buf[(MSG_LEN - 1)];         /* space for a message */
} SM_13_T;
#define SMTYPE_RESERVED13 13

typedef struct                         /* Reserved message */
{
   UBYTE_T MsgType;                    /* message type */
   BYTE_T ReplyID;                     /* reply identification */
   UWORD_T Buf[(MSG_LEN - 1)];         /* space for a message */
} SM_14_T;
#define SMTYPE_RESERVED14 14

typedef struct                         /* Reserved message */
{
   UBYTE_T MsgType;                    /* message type */
   BYTE_T ReplyID;                     /* reply identification */
   UWORD_T Buf[(MSG_LEN - 1)];         /* space for a message */
} SM_15_T;
#define SMTYPE_RESERVED15 15

/*****************************************************************************
                        Trace Defines & Structures
Each type of trace has a TRCB (Trace Control Block) associated with it.
TRTbl[] is an array on TRCBs, which is indexed by the trace type ID.
Trace types can be individually enabled and can have individual breaks
enabled, as defined by the Enable member.  Each time an enabled trace occurs
it's Count member is incremented and a trace entry is made in the trace
buffer.  Traces are enabled and disabled with kSetTrace().  Entries in
the trace buffer are made with kTrace().

Trace type ID numbers are defined below, and have the TRID_XX_ prefix, where
XX is replaced with one or two letters that identify the trace catagory.
TRID_U_ traces are user traces, kTrace() calls using these IDs
can be placed anywhere in the SCP.  TRID_KF_ traces are SCK function traces,
these are further identified by the name of the SCK function being traced.
TRID_KT_ traces are SCK defined task traces, these are further identified by
the name of the SCK task being traced.  Additional trace catagories may be
added in the future.

When a trace occurs, if Enable is set to (TR_EN | TR_BK) and Count == BkCount,
the normal trace entry is made, FuP->IFlag is set to IF_BK, and a loop is
entered until FuP->IFlag == IF_RDY.  This allows the SSCI to restart the
control program by writing IF_RDY into FuP->IFlag.

When a trace occurs, if Enable is set to (TR_EN | TR_BK | TR_TD) and
Count == BkCount, the normal trace entry is made and an "int 27h" instruction
is executed to invoke the Turbo Debugger kernel in firmware.  This freezes the
control program and allows a remote PC to debug the control program.  This
feature is not currently supported.
******************************************************************************/
/*----------------------------------------------------------------------------
                           Trace Table Size
The trace table has one TRCB (trace control block) for each possible trace
type.  TRTBL_SIZE is the number of TRCBs in the trace table, and is one
greater than the largest trace type ID number that can exist.
-----------------------------------------------------------------------------*/
/* #define TRTBL_SIZE - see below */
#define TRMSG_SIZE 28         /* size of trace message */
#define TRBUF_SIZE 64         /* number of 32 byte entries in trace buf */

/*----------------------------------------------------------------------------
                           User Trace IDs
The developer can place calls to kTrace() anywhere in the SCP, using any of
these defines as the trace identification.
-----------------------------------------------------------------------------*/
#define TRID_U_0 0x0
#define TRID_U_1 0x1
#define TRID_U_2 0x2
#define TRID_U_3 0x3
#define TRID_U_4 0x4
#define TRID_U_5 0x5
#define TRID_U_6 0x6
#define TRID_U_7 0x7

/*----------------------------------------------------------------------------
                            SCK Task and ISR Trace IDs
These are used to trace SCK defined tasks and ISRs.  These traces are
not directly related to SCK calls the SCP can make.
-----------------------------------------------------------------------------*/
#define TRID_KT_CTLINT 0x8    /* kCtlInt(), indicates SCK int occurred */
#define TRID_KT_CTLMSG 0x9    /* message in CtlQ dispatched by kTaskCtlQ() */
#define TRID_KT_CTLIFG 0xa    /* SCK int flag group dispatched by kTaskIFG()*/
#define TRID_KT_RESB 0xb                /* reserved */
#define TRID_KT_RESC 0xc                /* reserved */
#define TRID_KT_RESD 0xd                /* reserved */
#define TRID_KT_RESE 0xe                /* reserved */
#define TRID_KT_RESF 0xf                /* reserved */

/*----------------------------------------------------------------------------
                          SCK Function Trace IDs
These are used to trace SCK functions.
-----------------------------------------------------------------------------*/
#define TRID_KF_CREATETASK 0x10        /* kTaskCreate() */
#define TRID_KF_DELAY 0x11             /* kDelay() */
#define TRID_KF_LOCK 0x12              /* kLock() */
#define TRID_KF_UNLOCK 0x13            /* kUnLock() */
#define TRID_KF_CHANGEPRIO 0x14        /* kChangePriority() */
#define TRID_KF_DELETETASK 0x15        /* kDeleteTask() */

#define TRID_KF_INITSEM 0x16            /* kInitSem() */
#define TRID_KF_PENDSEM_NOSUS 0x17  /* kPendSem() call, did not suspend task */
#define TRID_KF_PENDSEM_SUS 0x18    /* kPendSem() call, task suspended */
#define TRID_KF_PENDSEM_RESTIME 0x19 /* kPendSem(), resume due to timeout */
#define TRID_KF_PENDSEM_RESPOST 0x1a /* kPendSem(), resume due to sem post */
#define TRID_KF_POSTSEM_NOPEND 0x1b    /* kPostSem(), no task pending */
#define TRID_KF_POSTSEM_PEND 0x1c      /* kPostSem(), task pending */
#define TRID_KF_POSTSEM_OVF 0x1d       /* kPostSem(), sem overflow */

#define TRID_KF_INITBOX 0x1e           /* kInitMBox() */
#define TRID_KF_PENDBOX_NOSUS 0x1f  /* kPendMBox() call, did not suspend task*/
#define TRID_KF_PENDBOX_SUS 0x20    /* kPendMBox() call, task suspended */
#define TRID_KF_PENDBOX_RESTIME 0x21 /* kPendMBox(), resume due to timeout */
#define TRID_KF_PENDBOX_RESPOST 0x22 /* kPendMBox(), resume due to mbox post */
#define TRID_KF_POSTBOX_FULL 0x23   /* kPostMBox(), error, mail box was full */
#define TRID_KF_POSTBOX_NOERR 0x24  /* kPostMBox(), no errors */

#define TRID_KF_INITQ 0x25          /* kInitQ() */
#define TRID_KF_PENDQ_NOSUS 0x26    /* kPendQ() call, did not suspend task */
#define TRID_KF_PENDQ_SUS 0x27      /* kPendQ() call, task suspended */
#define TRID_KF_PENDQ_RESTIME 0x28  /* kPendQ(), resume due to timeout */
#define TRID_KF_PENDQ_RESPOST 0x29  /* kPendQ(), resume due to queue post */
#define TRID_KF_POSTQ_FULL 0x2a     /* kPostQ(), error, queue was full */
#define TRID_KF_POSTQ_NOERR 0x2b    /* kPostQ(), no errors */

#define TRID_KF_SENDSYSQ_FULL 0x2c     /* kSendSysQMsg(), error, queue was full */
#define TRID_KF_SENDSYSQ_NOERR 0x2d    /* kSendSysQMsg(), no errors */
#define TRID_KF_SETIFG 0x2e            /* kSetIFG() */

#define TRID_KF_STARTTX 0x2f           /* kStartTx() */
#define TRID_KF_RESTARTTX 0x30         /* kRestartTx() */

/*                     ***** WARNING *****
   change this if more traces are added */
#define TRTBL_SIZE 0x31                /* size of TRTbl[] */

typedef struct
{
   UBYTE_T Enable;            /* enable flags, defined below */
   UWORD_T Count;             /* count of times this trace occurred */
   UWORD_T BkCount;           /* break when Count == BkCount */
} TRCB_T;

/* Trace enable flags */
#define TR_DIS 0x00           /* disable trace type */
#define TR_EN 0x01            /* enable trace type */
#define TR_BK 0x02            /* break when Count == BkCount */
#define TR_TD 0x04            /* invoke Turbo Debug on break */

/*----------------------------------------------------------------------------
                          Trace Message Descriptors
The trace buffer is a queue of TR_ENTRY_T descriptors.  TrBuf[] is an
array of these, and TrIx is the index of the next entry to put a trace in.
Each trace type has it's own trace message descriptor.  The TrMsgDesc member of
TR_ENTRY_T is a placeholder for the trace message descriptor   Most trace types
except those with IDs of TR_U_X have trace message descriptors defined below.
The TR_U_X types have their trace message descriptors defined by the user.
-----------------------------------------------------------------------------*/
typedef struct                         /* generic trace descriptor */
{
   WORD_T Ident;                       /* trace identifier */
   UWORD_T Count;                      /* count of times this trace occurred */
   UBYTE_T TrMsgDesc[TRMSG_SIZE];      /* space for trace message descriptor */
} TR_ENTRY_T;

typedef struct                         /* kTaskCreate() trace message */
{
   ULONG_T Task;
   UWORD_T Data;
   UWORD_T StkP;
   UWORD_T StkProbeP;
   UBYTE_T p;
} TR_CREATETASK_T;

typedef struct                         /* kSetIFG(), kTaskIFG() trace message*/
{
   UWORD_T Flags;                      /* mask parm or state of flag group */
   BYTE_T Group;                       /* parm, flag group number */
   UBYTE_T p;                          /* priority of current task */
} TR_IFG_T;

typedef struct                         /* kDelay() trace message */
{
   ULONG_T Time;                       /* time parameter */
   UBYTE_T p;                          /* priority of current task */
} TR_DELAY_T;

typedef struct                         /* kLock() trace message */
{
   UBYTE_T p;                          /* priority of current task */
} TR_LOCK_T;

typedef struct                         /* kChangePriority() trace message */
{
   UBYTE_T NewPrio;                    /* new priority parameter */
   UBYTE_T p;                          /* priority of current task */
} TR_CHANGEPRIO_T;

typedef struct                         /* kDeleteTask() trace message */
{
   UBYTE_T p;                          /* priority of current task */
} TR_DELETETASK_T;

typedef struct                         /* kDeleteTask() trace message */
{
   void *SemP;                         /* parm, ptr to semaphore */
   LONG_T Timeout;                     /* parm, timeout (kInitSem() only) */
   WORD_T Cnt;                         /* parm, sem count */
   UBYTE_T p;                          /* priority of current task */
} TR_SEM_T;

#define TR_BOX_MSGSIZE TRMSG_SIZE - 7
typedef struct                         /* kPendMBox() trace message */
{
   void *BoxP;                         /* parameter, ptr to mail box */
   LONG_T Timeout;                     /* parameter, timeout */
   UBYTE_T p;                          /* priority of current task */
   UBYTE_T Msg[TR_BOX_MSGSIZE];        /* message in mail box */
} TR_BOX_T;

#define TR_Q_MSGSIZE TRMSG_SIZE - 7
typedef struct                         /* kPendQ() trace message */
{
   void *pq;                           /* parameter, ptr to queue */
   LONG_T Timeout;                     /* parameter, timeout */
   UBYTE_T p;                          /* priority of current task */
   UBYTE_T Msg[TR_Q_MSGSIZE];          /* message in queue */
} TR_Q_T;

#define TR_MSGQ_MSGSIZE TRMSG_SIZE - 7
typedef struct                         /* kSendSysQMsg(), etc. trace message */
{
   WORD_T Size;                        /* parm, message size */
   WORD_T In;                          /* queue in ptr */
   WORD_T Out;                         /* queue out ptr */
   UBYTE_T p;                          /* priority of current task */
   UBYTE_T Msg[TR_MSGQ_MSGSIZE];       /* message in queue */
} TR_MSGQ_T;

typedef struct                         /* kCtlInt() trace message */
{
   BYTE_T Type;                        /* interrupt type */
   UBYTE_T p;                          /* priority of current task */
} TR_CTLINT_T;

typedef struct
{
   WORD_T Ln;                          /* line number */
   WORD_T n;                           /* SDB parameter */
   WORD_T Cnt;                         /* SDB count */
   WORD_T Status;                      /* line status */
   WORD_T Modes;                       /* line modes */
   UBYTE_T p;                          /* priority of current task */
} TR_STARTTX_T;

typedef struct
{
   WORD_T Ln;                          /* line number */
   WORD_T TxLocSDB;                    /* current TxLocSDB handle */
   WORD_T TxLocCnt;                    /* count of bytes remaining in current
                                          TxLlocSDB */
   WORD_T TxLocQ;                      /* first SDB handle on TxLocQ */
   UWORD_T Status;                     /* status member of line descriptor */
   UWORD_T Modes;                      /* modes member of line descriptor */
   UBYTE_T p;                          /* current task priority */
} TR_RESTARTTX_T;

#endif /* #ifdef SC_IP */
